from __future__ import annotations

import logging
from typing import Callable

from django.contrib import messages
from django.http.request import HttpRequest
from django.http.response import HttpResponse
from django.shortcuts import redirect
from django.template.loader import render_to_string
from django.urls import reverse

from impersonate_permissions.models import PermissionWindow

from .settings import DISPLAY_MESSAGES, EXPIRY_WARNING_THRESHOLD_MINS

logger = logging.getLogger(__name__)


def add_message_impersonating(request: HttpRequest, window: PermissionWindow) -> None:
    if not DISPLAY_MESSAGES:
        return
    if window.ttl.total_seconds() < (EXPIRY_WARNING_THRESHOLD_MINS * 60):
        level = messages.WARNING
    else:
        level = messages.INFO
    message = render_to_string(
        "impersonate_permissions/impersonating.tpl", context={"window": window}
    )
    messages.add_message(request, level, message)


def add_message_expired(request: HttpRequest, window: PermissionWindow) -> None:
    if not DISPLAY_MESSAGES:
        return
    message = render_to_string(
        "impersonate_permissions/expired.tpl", context={"window": window}
    )
    messages.add_message(request, messages.INFO, message)


class ImpersonatePermissionsMiddleware:
    """
    Extract and verify request tokens from incoming GET requests.

    This middleware is used to perform initial JWT verfication of
    link tokens.

    """

    def __init__(self, get_response: Callable):
        self.get_response = get_response

    def __call__(self, request: HttpRequest) -> HttpResponse:  # noqa: C901
        """
        Verify impersonation permissions.

        This call will fail hard if the request has no user attribute.
        """
        if not request.user.is_impersonate:
            return self.get_response(request)

        # don't interfere with this page, otherwise we get into loop
        if request.path == reverse("impersonate-stop"):
            return self.get_response(request)

        # the user being impersonated is in the permitted_users
        window = request.user.permission_windows.active().last()
        if window:
            add_message_impersonating(request, window)
            return self.get_response(request)

        add_message_expired(request, window)
        return redirect(reverse("impersonate-stop"))



from __future__ import annotations

from django.contrib import admin

from .models import PermissionWindow


class PermissionWindowAdmin(admin.ModelAdmin):

    list_display = (
        "user",
        "window_starts_at",
        "window_ends_at",
        "is_enabled",
        "is_active_",
    )
    search_fields = (
        "user__first_name",
        "user__last_name",
        "user__email",
        "user__username",
    )
    raw_id_fields = ("user",)
    readonly_fields = ("created_at",)

    def is_active_(self, obj: PermissionWindow) -> bool:
        return obj.is_active

    is_active_.boolean = True  # type: ignore


admin.site.register(PermissionWindow, PermissionWindowAdmin)



from os import getenv
from typing import Any, Callable

from django.conf import settings


def _setting(key: str, default: Any, cast: Callable) -> Any:
    key = f"IMPERSONATE_PERMISSIONS_{key}"
    return cast(getenv(key) or getattr(settings, key, default))


# Default expiry, in minutes
DEFAULT_EXPIRY_MINS: int = _setting("DEFAULT_EXPIRY", 60, int)

DISPLAY_MESSAGES: bool = _setting("DISPLAY_MESSAGES", True, bool)

EXPIRY_WARNING_THRESHOLD_MINS: int = _setting("EXPIRY_WARNING_THRESHOLD_MINS", 10, int)



from __future__ import annotations

import datetime

from django.conf import settings
from django.contrib.auth import get_user_model
from django.db import models, transaction
from django.http import HttpRequest
from django.utils import timezone
from django.utils.translation import gettext_lazy as _

from .settings import DEFAULT_EXPIRY_MINS

User = get_user_model()


def default_expiry() -> datetime.datetime:
    """Return a timestamp based on DEFAULT_EXPIRY."""
    return timezone.now() + datetime.timedelta(minutes=DEFAULT_EXPIRY_MINS)


def permitted_users(request: HttpRequest) -> models.QuerySet:
    """Return users who can be impersonated."""
    user_ids = PermissionWindow.objects.active().values_list("user_id", flat=True)
    return User.objects.filter(id__in=user_ids).order_by("first_name", "last_name")


class PermissionWindowQuerySet(models.QuerySet):
    def active(self) -> PermissionWindowQuerySet:
        """Return active and enabled PermissionWindows."""
        now = timezone.now()
        return self.filter(
            window_starts_at__lte=now, window_ends_at__gte=now, is_enabled=True
        ).order_by("window_starts_at", "window_ends_at")

    def disable(self) -> None:
        """Disable all objects in queryset."""
        self.update(is_enabled=False)


class PermissionWindowManager(models.Manager):
    @transaction.atomic
    def create(self, user: settings.AUTH_USER_MODEL, **kwargs: str) -> PermissionWindow:
        """Create new PermissionWindow and disable any existing windows."""
        user.permission_windows.active().disable()
        return super().create(user=user, **kwargs)


class PermissionWindow(models.Model):
    """
    Maintain user permission for impersonation.

    This class stores a time-based window within which a user has granted
    permission for a staff member to access their account via impersonation.

    """

    user = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        on_delete=models.CASCADE,
        related_name="permission_windows",
    )
    window_starts_at = models.DateTimeField(
        default=timezone.now, help_text=_("When the permission window begins.")
    )
    window_ends_at = models.DateTimeField(
        default=default_expiry, help_text=_("When the permission window ends.")
    )
    is_enabled = models.BooleanField(
        default=True, help_text=_("Kill switch for permission window.")
    )
    created_at = models.DateTimeField(
        default=timezone.now, help_text=_("When the database record was created.")
    )

    objects = PermissionWindowManager.from_queryset(PermissionWindowQuerySet)()

    def __str__(self) -> str:
        return f"Impersonate permissions window [{self.id}] for {self.user}"

    def __repr__(self) -> str:
        return f"<PermisssionWindow id={self.id} user_id={self.user_id}>"

    @property
    def is_active(self) -> bool:
        """Return True if the window is open and permission is enabled."""
        return self.is_enabled and (
            self.window_starts_at < timezone.now() < self.window_ends_at
        )

    @property
    def ttl(self) -> datetime.timedelta:
        """Return time to expiry."""
        return self.window_ends_at - timezone.now()

    def disable(self) -> None:
        """Disable the window by setting enabled to False."""
        self.is_enabled = False
        self.save()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['impersonate_permissions', 'impersonate_permissions.migrations']

package_data = \
{'': ['*'], 'impersonate_permissions': ['templates/impersonate_permissions/*']}

install_requires = \
['django-impersonate>=1.5.1,<2.0.0', 'django>=2.2,<4.0']

setup_kwargs = {
    'name': 'django-impersonate-permissions',
    'version': '0.1.2.dev0',
    'description': 'Django app that tracks user permssions when using django-impersonate.',
    'long_description': '# Django Impersonate Permissions\n\nAdd ability to control impersonate permissions.\n',
    'author': 'YunoJuno',
    'author_email': 'code@yunojuno.com',
    'maintainer': 'YunoJuno',
    'maintainer_email': 'code@yunojuno.com',
    'url': 'https://github.com/yunojuno/django-impersonate-permissions',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



# Generated by Django 3.0.8 on 2020-07-18 12:33

import django.db.models.deletion
import django.utils.timezone
from django.conf import settings
from django.db import migrations, models

import impersonate_permissions.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="PermissionWindow",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "window_starts_at",
                    models.DateTimeField(
                        default=django.utils.timezone.now,
                        help_text="When the permission window begins.",
                    ),
                ),
                (
                    "window_ends_at",
                    models.DateTimeField(
                        default=impersonate_permissions.models.default_expiry,
                        help_text="When the permission window ends.",
                    ),
                ),
                (
                    "is_enabled",
                    models.BooleanField(
                        default=True, help_text="Kill switch for permission window."
                    ),
                ),
                (
                    "created_at",
                    models.DateTimeField(
                        default=django.utils.timezone.now,
                        help_text="When the database record was created.",
                    ),
                ),
                (
                    "user",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="permission_windows",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
        ),
    ]



from typing import Dict, Optional, Union

from django.conf import settings
from django.http import HttpRequest


def impersonation(
    request: HttpRequest,
) -> Dict[str, Union[bool, Optional[settings.AUTH_USER_MODEL]]]:
    """Add impersonate info to template context."""
    if request.user.is_impersonate:
        return {
            "is_impersonate": True,
            "impersonator": request.real_user,
            "impersonating": request.user,
        }
    return {"is_impersonate": False, "impersonator": None, "impersonating": None}
